#!/usr/bin/perl
# -----------------------------------------------------------------------------
# TWC
# Treuwert-Computer GmbH
# Schlobergring 9
# 79098 Freiburg
#
# Author: Klaus Reger <K.Reger@twc.de>
#
# -----------------------------------------------------------------------------
# Program-Data
# -----------------------------------------------------------------------------
# Project  :
# System   :
# Program  :
# Module   : $RCSfile: TestBase.t,v $
# Version  : $Revision: 1.1 $
# Date     : $Date: 2002/02/28 15:23:50 $
# State    : $State: Exp $
#
# Description:
#   This module handles all the things of WWWdb, which have
#   to do with Lists
#
# Change-log (See at end of this file)
#
# --- Known Bugs -------------
# please look at the BUGS-File
# --- TODO -------------------
# please look at the TODO-File
# ----------------------------------------------------------------------------



use WWWdb::Base;


# --------------------------------------------------------------------
# Test_0
# --------------------------------------------------------------------
package Test_0;
use vars qw(%hAttribs);

@Test_0::ISA = qw(Base);

{
   {

      %hAttribs =
          (
           t0_pub_rw  => ["i", "public",     "rw", 0],
           t0_pro_rw  => ["i", "protected",  "rw", 0],
           t0_pri_rw  => ["i", "private",    "rw", 0],
           t0_pub_r   => ["i", "public",     "r",  0],
           t0_pro_r   => ["i", "protected",  "r",  0],
           t0_pri_r   => ["i", "private",    "r",  0],
           );
   }

   # --- constructor ---------------------------------------------------
   sub Test_0::new()
   {
      my ($cPkgPI)      = shift;

      my $cClass   = ref($cPkgPI) || $cPkgPI;

      my $hSelf   = $cClass->SUPER::new(@_);

      foreach (@ISA)
      {
         my $cInitMethod = "${_}::_init";

         $hSelf->$cInitMethod($_, @_);
      }

      return $hSelf;
   }
}



# --------------------------------------------------------------------
# Test_A
# --------------------------------------------------------------------
package Test_A;
use vars qw(%hAttribs);

@Test_A::ISA = qw(Base);

{
   {
      # private, protected, public
      %hAttribs =
          (
           ta_pub_rw  => ["c", "public",     "rw", "a"],
           ta_pro_rw  => ["c", "protected",  "rw", "a"],
           ta_pri_rw  => ["c", "private",    "rw", "a"],
           ta_pub_r   => ["c", "public",     "r",  "a"],
           ta_pro_r   => ["c", "protected",  "r",  "a"],
           ta_pri_r   => ["c", "private",    "r",  "a"],
           );
   }

   # --- constructor ---------------------------------------------------
   sub Test_A::new()
   {
      my ($cPkgPI)      = shift;

      my $cClass   = ref($cPkgPI) || $cPkgPI;

      my $hSelf   = $cClass->SUPER::new(@_);

      foreach (@ISA)
      {
         my $cInitMethod = "${_}::_init";

         $hSelf->$cInitMethod($_, @_);
      }

      return $hSelf;
   }
}



# --------------------------------------------------------------------
# Test_1
# --------------------------------------------------------------------
package Test_1;
use vars qw(%hMethods %hAttribs);

@Test_1::ISA = qw(Test_0);

{
      # private, protected, public
      %hAttribs =
          (
           t1_pub_rw  => ["c", "public",     "rw", 1],
           t1_pro_rw  => ["c", "protected",  "rw", 1],
           t1_pri_rw  => ["c", "private",    "rw", 1],
           t1_pub_r   => ["c", "public",     "r",  1],
           t1_pro_r   => ["c", "protected",  "r",  1],
           t1_pri_r   => ["c", "private",    "r",  1],
           );
}



# --- constructor ---------------------------------------------------
sub Test_1::new()
{
   my ($cPkgPI)      = shift;

   my $cClass   = ref($cPkgPI) || $cPkgPI;

   my $hSelf   = $cClass->SUPER::new(@_);

   foreach (@ISA)
   {
         my $cInitMethod = "${_}::_init";

         $hSelf->$cInitMethod($_, @_);
   }

   return $hSelf;
}


# --------------------------------------------------------------------
# Test_2
# --------------------------------------------------------------------
package Test_2;
use vars qw(%hMethods %hAttribs);

@Test_2::ISA = qw(Test_1);

{
      # private, protected, public
      %hAttribs =
          (
           t2_pub_rw  => ["o", "public",     "rw", 2],
           t2_pro_rw  => ["o", "protected",  "rw", 2],
           t2_pri_rw  => ["o", "private",    "rw", 2],
           t2_pub_r   => ["o", "public",     "r",  2],
           t2_pro_r   => ["o", "protected",  "r",  2],
           t2_pri_r   => ["o", "private",    "r",  2],
           );
}



# --- constructor ---------------------------------------------------
sub Test_2::new()
{
   my ($cPkgPI)      = shift;

   my $cClass   = ref($cPkgPI) || $cPkgPI;

   my $hSelf   = $cClass->SUPER::new(@_);

   foreach (@ISA)
   {
         my $cInitMethod = "${_}::_init";

         $hSelf->$cInitMethod($_, @_);
   }

   return $hSelf;
}



# --------------------------------------------------------------------
# Test_0A
# --------------------------------------------------------------------
package Test_0A;
use vars qw(%hMethods %hAttribs);

@Test_0A::ISA = qw(Test_0 Test_A);

{
      # private, protected, public
      %hAttribs =
          ();
}



# --- constructor ---------------------------------------------------
sub Test_0A::new()
{
   my ($cPkgPI)      = shift;

   my $cClass   = ref($cPkgPI) || $cPkgPI;

   my $hSelf   = $cClass->SUPER::new(@_);

   foreach (@ISA)
   {
         my $cInitMethod = "${_}::_init";

         $hSelf->$cInitMethod($_, @_);
   }

   return $hSelf;
}





package main;


$oTest_0 = Test_0->new(t0_pub_rw  => -10,
                       t0_pro_rw  => -20,
                       t0_pri_rw  => -30,
                       t0_pub_r   => -40,
                       t0_pro_r   => -50,
                       t0_pri_r   => -60,
                       t0_not_ex  => -99);

eval { $oTest_0->sett0_pub_rw();   } or warn $@;
eval { $oTest_0->sett0_not_ex(999);} or warn $@;
eval { $oTest_0->sett0_pub_rw(10); } or warn $@;
eval { $oTest_0->sett0_pro_rw(20); } or warn $@;
eval { $oTest_0->sett0_pri_rw(30); } or warn $@;
eval { $oTest_0->sett0_pub_r(40);  } or warn $@;
eval { $oTest_0->sett0_pro_r(50);  } or warn $@;
eval { $oTest_0->sett0_pri_r(60);  } or warn $@;

eval { $oTest_0->gett0_pub_rw(10); } or warn $@;
eval { $oTest_0->gett0_pub_rw();   } or warn $@;
eval { $oTest_0->gett0_pro_rw();   } or warn $@;
eval { $oTest_0->gett0_pri_rw();   } or warn $@;
eval { $oTest_0->gett0_pub_r();    } or warn $@;
eval { $oTest_0->gett0_pro_r();    } or warn $@;
eval { $oTest_0->gett0_pri_r();    } or warn $@;

$oTest_0->Dump();

$oTest_0Clone = Test_0->new($oTest_0);
$oTest_0Clone->Dump();



$oTest_1 = Test_1->new(t0_pub_rw  => -10,
                       t0_pro_rw  => -20,
                       t0_pri_rw  => -30,
                       t0_pub_r   => -40,
                       t0_pro_r   => -50,
                       t0_pri_r   => -60,
                       t1_pub_rw  => -11,
                       t1_pro_rw  => -21,
                       t1_pri_rw  => -31,
                       t1_pub_r   => -41,
                       t1_pro_r   => -51,
                       t1_pri_r   => -61);

eval { $oTest_1->sett0_pub_rw();   } or warn $@;
eval { $oTest_1->sett0_pub_rw(10); } or warn $@;
eval { $oTest_1->sett0_pro_rw(20); } or warn $@;
eval { $oTest_1->sett0_pri_rw(30); } or warn $@;
eval { $oTest_1->sett0_pub_r(40);  } or warn $@;
eval { $oTest_1->sett0_pro_r(50);  } or warn $@;
eval { $oTest_1->sett0_pri_r(60);  } or warn $@;

eval { $oTest_1->gett0_pub_rw(10); } or warn $@;
eval { $oTest_1->gett0_pub_rw();   } or warn $@;
eval { $oTest_1->gett0_pro_rw();   } or warn $@;
eval { $oTest_1->gett0_pri_rw();   } or warn $@;
eval { $oTest_1->gett0_pub_r();    } or warn $@;
eval { $oTest_1->gett0_pro_r();    } or warn $@;
eval { $oTest_1->gett0_pri_r();    } or warn $@;

eval { $oTest_1->sett1_pub_rw();   } or warn $@;
eval { $oTest_1->sett1_pub_rw(11); } or warn $@;
eval { $oTest_1->sett1_pro_rw(21); } or warn $@;
eval { $oTest_1->sett1_pri_rw(31); } or warn $@;
eval { $oTest_1->sett1_pub_r(41);  } or warn $@;
eval { $oTest_1->sett1_pro_r(51);  } or warn $@;
eval { $oTest_1->sett1_pri_r(61);  } or warn $@;

eval { $oTest_1->gett1_pub_rw(11); } or warn $@;
eval { $oTest_1->gett1_pub_rw();   } or warn $@;
eval { $oTest_1->gett1_pro_rw();   } or warn $@;
eval { $oTest_1->gett1_pri_rw();   } or warn $@;
eval { $oTest_1->gett1_pub_r();    } or warn $@;
eval { $oTest_1->gett1_pro_r();    } or warn $@;
eval { $oTest_1->gett1_pri_r();    } or warn $@;

$oTest_1->Dump();






$oTest_2 = Test_2->new(t0_pub_rw  => -10,
                       t0_pro_rw  => -20,
                       t0_pri_rw  => -30,
                       t0_pub_r   => -40,
                       t0_pro_r   => -50,
                       t0_pri_r   => -60,
                       t2_pub_rw  => -11,
                       t2_pro_rw  => -21,
                       t2_pri_rw  => -31,
                       t2_pub_r   => -41,
                       t2_pro_r   => -51,
                       t2_pri_r   => -61);

eval { $oTest_2->sett0_pub_rw();   } or warn $@;
eval { $oTest_2->sett0_pub_rw(10); } or warn $@;
eval { $oTest_2->sett0_pro_rw(20); } or warn $@;
eval { $oTest_2->sett0_pri_rw(30); } or warn $@;
eval { $oTest_2->sett0_pub_r(40);  } or warn $@;
eval { $oTest_2->sett0_pro_r(50);  } or warn $@;
eval { $oTest_2->sett0_pri_r(60);  } or warn $@;

eval { $oTest_2->gett0_pub_rw(10); } or warn $@;
eval { $oTest_2->gett0_pub_rw();   } or warn $@;
eval { $oTest_2->gett0_pro_rw();   } or warn $@;
eval { $oTest_2->gett0_pri_rw();   } or warn $@;
eval { $oTest_2->gett0_pub_r();    } or warn $@;
eval { $oTest_2->gett0_pro_r();    } or warn $@;
eval { $oTest_2->gett0_pri_r();    } or warn $@;

eval { $oTest_2->sett2_pub_rw();   } or warn $@;
eval { $oTest_2->sett2_pub_rw(12); } or warn $@;
eval { $oTest_2->sett2_pro_rw(22); } or warn $@;
eval { $oTest_2->sett2_pri_rw(32); } or warn $@;
eval { $oTest_2->sett2_pub_r(42);  } or warn $@;
eval { $oTest_2->sett2_pro_r(52);  } or warn $@;
eval { $oTest_2->sett2_pri_r(62);  } or warn $@;

eval { $oTest_2->gett2_pub_rw(12); } or warn $@;
eval { $oTest_2->gett2_pub_rw();   } or warn $@;
eval { $oTest_2->gett2_pro_rw();   } or warn $@;
eval { $oTest_2->gett2_pri_rw();   } or warn $@;
eval { $oTest_2->gett2_pub_r();    } or warn $@;
eval { $oTest_2->gett2_pro_r();    } or warn $@;
eval { $oTest_2->gett2_pri_r();    } or warn $@;

$oTest_2->Dump();

$oTest_2->ShowMethods();



$oTest_0A = Test_0A->new(t0_pub_rw  => -10,
                         t0_pro_rw  => -20,
                         t0_pri_rw  => -30,
                         t0_pub_r   => -40,
                         t0_pro_r   => -50,
                         t0_pri_r   => -60,
                         t0_not_ex  => -99,
                         ta_pub_rw  => "-1a",
                         ta_pro_rw  => "-2a",
                         ta_pri_rw  => "-3a",
                         ta_pub_r   => "-4a",
                         ta_pro_r   => "-5a",
                         ta_pri_r   => "-6a",
                         ta_not_ex  => "-9a");

eval { $oTest_0A->sett0_pub_rw();   } or warn $@;
eval { $oTest_0A->sett0_not_ex(999);} or warn $@;
eval { $oTest_0A->sett0_pub_rw(10); } or warn $@;
eval { $oTest_0A->sett0_pro_rw(20); } or warn $@;
eval { $oTest_0A->sett0_pri_rw(30); } or warn $@;
eval { $oTest_0A->sett0_pub_r(40);  } or warn $@;
eval { $oTest_0A->sett0_pro_r(50);  } or warn $@;
eval { $oTest_0A->sett0_pri_r(60);  } or warn $@;

eval { $oTest_0A->gett0_pub_rw(10); } or warn $@;
eval { $oTest_0A->gett0_pub_rw();   } or warn $@;
eval { $oTest_0A->gett0_pro_rw();   } or warn $@;
eval { $oTest_0A->gett0_pri_rw();   } or warn $@;
eval { $oTest_0A->gett0_pub_r();    } or warn $@;
eval { $oTest_0A->gett0_pro_r();    } or warn $@;
eval { $oTest_0A->gett0_pri_r();    } or warn $@;

$oTest_0A->Dump();

$oTest_0A->ShowMethods();

exit 0;
